/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Definitions
    Tx - Abbreviation of "Transaction".

Explicit Transacitons
    The User-Program may explicitly start & commit transactions.  This is useful when 
    the User-Program wishes all or none of actions to be made permanent.  If the User-
    Program detects an error, it may undo / rollback all since the start transaction
    command.
    
    An example of one such use is in an online ordering system for Airplane Tickets.  
    Lets say a user places an order for one airline ticket.  The system proceeds to 
    charge the credit card and reserve the seat.  If an error occurs while reserving
    the seat within the application, the credit card charge can be easily undone with
    the system.

    NOTE:  Rolling back changes affects the persistent datastore.  It does not 
           affect variables & DDOs in local memory.  The User-Application may 
           need to refresh all data in memory with the information from the 
           persistent datastore.
    
    Transaction
    Operations    Function            Description
    ------------  ------------------  ----------------------------------------------------------
    Start         startTransaction()  Start a new transaction
    ............................................................................................
    Rollback      rollback()          Undo / rollback all changes on the persistent datastore
                                      since the startTransaction() call.  Variables & DDOs in
                                      local memory will not be affected and should be considered
                                      invalid.
    ............................................................................................
    Commit        commit()            Commit changes / Make changes permanent.
    --------------------------------------------------------------------------------------------

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: STransactionsICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Explicit Transactions, Starting Transactions, Rolling Back Transactions,
 *                    Committing Traansactions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Explicit Transactions
 *                    Start Transaction
 *                    Rollback Transaction
 *                    Commit Transaction
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java STransactionsICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SSearchICM.java
 ************************************************************************************************/
public class STransactionsICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java STransactionsICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  STransactionsICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-----------------------------------------------------------------
			// Start an Explicit Transaction
			//-----------------------------------------------------------------
			System.out.println("Starting an explicit transaction...");
						
				dsICM.startTransaction();
						
			System.out.println("Started an explicit transaction.");

			//----------------------------------------------------------------
			// Do the something within transaction..
			//----------------------------------------------------------------
            // Perform operations within a transaction, such as creating
            // a new Item.
            System.out.println("Creating an Item...");

	            DKDDO ddoDocument = dsICM.createDDO("S_simple", DKConstant.DK_CM_DOCUMENT);
	            ddoDocument.add(); // Save the DDO in the persistent datastore.

	            String itemId = ((DKPidICM)ddoDocument.getPidObject()).getItemId();  // Save Item ID for later use.
	            System.out.println("Created Item '"+itemId+"'.");

            System.out.println("Created an Item.");
						
			//-----------------------------------------------------------------
    		// Rollback our changes.
			//-----------------------------------------------------------------
			System.out.println("Rolling back / Undoing changes...");
						
				dsICM.rollback();
						
			System.out.println("Rolled back changes.");

    		//-----------------------------------------------------------------
	        // Proving transaction was undone
    		//-----------------------------------------------------------------
	        // The item should not be added to the datastore.
	        // We can search for the Item ID, and no results should be found.
	        //
	        // For more information on Search, please refer to SSearchICM.
	        System.out.println("Proving transaction was undone...");
	        
                String query = "/*[@ITEMID=\""+itemId+"\"]";  // Looks like: /*[@ITEMID=\"A1001001A01J09B00241C95000\"];
                
                // Specify Search / Query Options
                DKNVPair options[] = new DKNVPair[3];
                options[0] = new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "1");                                          // Specify max using a string value.
                options[1] = new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE,    new Integer(DKConstant.DK_CM_CONTENT_IDONLY)); // Specify any Retrieval Options desired.
                options[2] = new DKNVPair(DKConstant.DK_CM_PARM_END,         null);                                         // Must mark the end of the NVPair

                DKResults results = (DKResults)dsICM.evaluate(query, DKConstant.DK_CM_XQPE_QL_TYPE, options);
                
                // No results should be found.
                SSearchICM.printResults(results);  // Print Results Using Wrapper in SSearchICM.
                if(results.cardinality()>0)
                    throw new Exception("Item(s) were found that match the Item ID.  The transaction was not undone.");

	        System.out.println("Proved transaction was undone.");

            //-----------------------------------------------------------------
			// Start another Explicit Transaction
			//-----------------------------------------------------------------
			System.out.println("Starting an explicit transaction...");
						
				dsICM.startTransaction();
						
			System.out.println("Started an explicit transaction.");

			//----------------------------------------------------------------
			// Do the something within transaction..
			//----------------------------------------------------------------
            // Perform operations within a transaction, such as creating
            // a new Item.
            System.out.println("Creating an Item...");

	            ddoDocument = dsICM.createDDO("S_simple", DKConstant.DK_CM_DOCUMENT);
	            ddoDocument.add(); // Save the DDO in the persistent datastore.

	            itemId = ((DKPidICM)ddoDocument.getPidObject()).getItemId();  // Save Item ID for later use.
	            System.out.println("Created Item '"+itemId+"'.");

            System.out.println("Created an Item.");
						
			//-----------------------------------------------------------------
    		// Commit Changes
			//-----------------------------------------------------------------
			// Make all changes permanent.
			System.out.println("Committing Changes / Making Changes Permanent...");
						
				dsICM.commit();
						
			System.out.println("Committed Changes / Made Changes Permanent.");
                
            //-------------------------------------------------------------
            // Clean Up Database After Sample
            //-------------------------------------------------------------
            // This sample will clean up / undo everything that stored
            // in the database.
            System.out.println("Cleaning Up...");

                ddoDocument.del();

            System.out.println("Cleaned Up.");
            
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class STransactionsICM
              
